@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: in_mad.c,v 1.23 2001/04/11 19:40:59 rob Exp $
+ * $Id: in_mad.c,v 1.35 2001/11/09 03:46:13 rob Exp $
  */
 
 # ifdef HAVE_CONFIG_H
@@ -30,16 +30,24 @@
 # include <prsht.h>
 # include <wininet.h>
 # include <stdio.h>
+# include <io.h>
+# include <fcntl.h>
 # include <stdarg.h>
 # include <string.h>
 # include <locale.h>
 # include <math.h>
 
+# ifndef M_PI
+#  define M_PI  3.14159265358979323846264338328
+# endif
+
 # include "resource.h"
 # include "messages.h"
 
 # include "in2.h"
 # include "mad.h"
+# include "id3tag.h"
+# include "xing.h"
 
 # define PLUGIN_VERSION		VERSION ""
 
@@ -108,6 +116,7 @@ static struct state {
   mad_fixed_t attenuation;	/* attenuation factor */
   int equalizer;		/* using equalizer */
   mad_fixed_t eqfactor[32];	/* equalizer settings */
+  struct xing xing;		/* Xing VBR header */
   struct stats stats;		/* statistics */
 } state;
 
@@ -389,9 +398,14 @@ static
 void apply_config(void)
 {
   module.FileExtensions = conf_enabled ?
+# if 1
+    "MP3;MP2;MP1\0" "MPEG Audio Files (*.MP3;*.MP2;*.MP1)\0"
+# else
     "MP3\0" "MPEG Audio Layer III files (*.MP3)\0"
     "MP2\0" "MPEG Audio Layer II files (*.MP2)\0"
-    "MP1\0" "MPEG Audio Layer I files (*.MP1)\0" : "";
+    "MP1\0" "MPEG Audio Layer I files (*.MP1)\0"
+# endif
+    : "";
 }
 
 static
@@ -492,6 +506,10 @@ BOOL config_dialog(HWND dialog, UINT message,
     if (conf_enabled)
       CheckDlgButton(dialog, IDC_CONF_ENABLED, BST_CHECKED);
 
+    PostMessage(dialog, WM_COMMAND,
+		MAKELONG(IDC_CONF_ENABLED, BN_CLICKED),
+		(LPARAM) GetDlgItem(dialog, IDC_CONF_ENABLED));
+
     /* Title Format */
 
     SendDlgItemMessage(dialog, IDC_TITLE_FORMAT, EM_SETLIMITTEXT,
@@ -666,23 +684,24 @@ void show_config(HWND parent)
 static
 void show_about(HWND parent)
 {
-  MessageBox(parent,
-	     "MPEG Audio Decoder version " MAD_VERSION "\n"
-	     "Winamp plug-in version " PLUGIN_VERSION "\n\n"
+  char const about[] =
+    "MPEG Audio Decoder version " MAD_VERSION "\n"
+    "ID3 Tag version " ID3_VERSION "\n"
+    "Winamp plug-in version " PLUGIN_VERSION "\n\n"
 
-	     "Copyright \xA9 " MAD_PUBLISHYEAR " " MAD_AUTHOR "\n\n"
+    "Copyright \xa9 " MAD_PUBLISHYEAR " " MAD_AUTHOR "\n\n"
 
-  "This program is free software; you can redistribute it and/or modify it\n"
-  "under the terms of the GNU General Public License as published by\n"
-  "the Free Software Foundation; either version 2 of the License, or (at\n"
-  "your option) any later version.\n\n"
+    "This program is free software; you can redistribute it and/or modify it\n"
+    "under the terms of the GNU General Public License as published by\n"
+    "the Free Software Foundation; either version 2 of the License, or (at\n"
+    "your option) any later version.\n\n"
 
-  "This program is distributed in the hope that it will be useful, but\n"
-  "WITHOUT ANY WARRANTY; without even the implied warranty of\n"
-  "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"
-  "See the GNU General Public License for more details.",
+    "This program is distributed in the hope that it will be useful, but\n"
+    "WITHOUT ANY WARRANTY; without even the implied warranty of\n"
+    "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"
+    "See the GNU General Public License for more details.";
 
-	     "About MAD Plug-in", MB_ICONINFORMATION | MB_OK);
+  MessageBox(parent, about, "About MAD Plug-in", MB_ICONINFORMATION | MB_OK);
 }
 
 static
@@ -769,16 +788,17 @@ void input_close(struct input *input)
   }
 }
 
+# if 0
 static
 int input_skip(struct input *input, unsigned long count)
 {
   if (input_seek(input, count, FILE_CURRENT) == -1) {
-    unsigned char buffer[256];
+    unsigned char sink[256];
     unsigned long len;
 
     while (count) {
-      len = input_read(input, buffer,
-		       count < sizeof(buffer) ? count : sizeof(buffer));
+      len = input_read(input, sink,
+		       count < sizeof(sink) ? count : sizeof(sink));
 
       if (len == -1)
 	return -1;
@@ -791,135 +811,7 @@ int input_skip(struct input *input, unsigned long count)
 
   return 0;
 }
-
-static
-int id3v2_tag(unsigned char const *buffer, unsigned long buflen,
-	      struct input *input, unsigned long *tagsize)
-{
-  unsigned char header_data[10];
-  unsigned char const *header;
-  struct {
-    unsigned int major;
-    unsigned int revision;
-  } version;
-  unsigned int flags;
-  unsigned long size, count, len;
-  unsigned char *ptr, *end, *tag = 0;
-  int result = 0;
-
-  enum {
-    FLAG_UNSYNC       = 0x0080,
-    FLAG_EXTENDED     = 0x0040,
-    FLAG_EXPERIMENTAL = 0x0020,
-
-    FLAG_UNKNOWN      = 0x001f
-  };
-
-  if (buflen < 10) {
-    header = header_data;
-
-    memcpy(header_data, buffer, buflen);
-
-    for (ptr = header_data + buflen, count = 10 - buflen;
-	 count; count -= len, ptr += len) {
-      len = input_read(input, ptr, count);
-
-      if (len == 0 || len == -1)
-	goto fail;
-    }
-
-    buffer += buflen;
-    buflen  = 0;
-  }
-  else {
-    header = buffer;
-
-    buffer += 10;
-    buflen -= 10;
-  }
-
-  if (header[3] == 0xff || header[4] == 0xff ||
-      (header[6] & 0x80) || (header[7] & 0x80) ||
-      (header[8] & 0x80) || (header[9] & 0x80))
-    goto fail;
-
-  version.major    = header[3];
-  version.revision = header[4];
-
-  flags = header[5];
-
-  /* high bit is not used */
-  size = (header[6] << 21) | (header[7] << 14) |
-         (header[8] <<  7) | (header[9] <<  0);
-
-  *tagsize = 10 + size;
-
-  if (version.major < 2 || version.major > 3)
-    goto abort;
-
-  if (size == 0)
-    goto done;
-
-  tag = LocalAlloc(0, size);
-  if (tag == 0)
-    goto abort;
-
-  if (buflen < size) {
-    memcpy(tag, buffer, buflen);
-
-    for (ptr = tag + buflen, count = size - buflen;
-	 count; count -= len, ptr += len) {
-      len = input_read(input, ptr, count);
-
-      if (len == 0 || len == -1)
-	goto fail;
-    }
-
-    buffer += buflen;
-    buflen  = 0;
-  }
-  else {
-    memcpy(tag, buffer, size);
-
-    buffer += size;
-    buflen -= size;
-  }
-
-  /* undo unsynchronisation */
-
-  if (flags & FLAG_UNSYNC) {
-    unsigned char *new = tag;
-
-    count = size;
-
-    for (ptr = tag; ptr < tag + size - 1; ++ptr) {
-      *new++ = *ptr;
-
-      if (ptr[0] == 0xff && ptr[1] == 0x00)
-	--count, ++ptr;
-    }
-
-    size = count;
-  }
-
-  ptr = tag;
-  end = tag + size;
-
-  goto done;
-
- fail:
-  result = -1;
-
- done:
-  if (tag)
-    LocalFree(tag);
-
-  return result;
-
- abort:
-  /* skip the unread part of the tag */
-  return (size > buflen) ? input_skip(input, size - buflen) : 0;
-}
+# endif
 
 static
 void mono_filter(struct mad_frame *frame)
@@ -982,6 +874,86 @@ void equalizer_filter(struct mad_frame *frame, mad_fixed_t eqfactor[32])
 }
 # endif
 
+# define NEW_DITHER
+# ifdef NEW_DITHER
+struct dither {
+  mad_fixed_t error[3];
+  mad_fixed_t random;
+};
+
+/*
+ * NAME:	prng()
+ * DESCRIPTION:	32-bit pseudo-random number generator
+ */
+static inline
+unsigned long prng(unsigned long state)
+{
+  return (state * 0x0019660dL + 0x3c6ef35fL) & 0xffffffffL;
+}
+
+static inline
+signed long linear_dither(unsigned int bits, mad_fixed_t sample,
+			  struct dither *dither, unsigned long *clipped,
+			  mad_fixed_t *clipping)
+{
+  unsigned int scalebits;
+  mad_fixed_t output, mask, random;
+
+  enum {
+    MIN = -MAD_F_ONE,
+    MAX =  MAD_F_ONE - 1
+  };
+
+  /* noise shape */
+  sample += dither->error[0] - dither->error[1] + dither->error[2];
+
+  dither->error[2] = dither->error[1];
+  dither->error[1] = dither->error[0] / 2;
+
+  /* bias */
+  output = sample + (1L << (MAD_F_FRACBITS + 1 - bits - 1));
+
+  scalebits = MAD_F_FRACBITS + 1 - bits;
+  mask = (1L << scalebits) - 1;
+
+  /* dither */
+  random  = prng(dither->random);
+  output += (random & mask) - (dither->random & mask);
+
+  dither->random = random;
+
+  /* clip */
+  if (output > MAX) {
+    ++*clipped;
+    if (output - MAX > *clipping)
+      *clipping = output - MAX;
+
+    output = MAX;
+
+    if (sample > MAX)
+      sample = MAX;
+  }
+  else if (output < MIN) {
+    ++*clipped;
+    if (MIN - output > *clipping)
+      *clipping = MIN - output;
+
+    output = MIN;
+
+    if (sample < MIN)
+      sample = MIN;
+  }
+
+  /* quantize */
+  output &= ~mask;
+
+  /* error feedback */
+  dither->error[0] = sample - output;
+
+  /* scale */
+  return output >> scalebits;
+}
+# else
 static inline
 signed long linear_dither(unsigned int bits, mad_fixed_t sample,
 			  mad_fixed_t *error, unsigned long *clipped,
@@ -1021,6 +993,7 @@ signed long linear_dither(unsigned int bits, mad_fixed_t sample,
   /* scale */
   return quantized >> (MAD_F_FRACBITS + 1 - bits);
 }
+# endif
 
 static
 unsigned int pack_pcm(unsigned char *data, unsigned int nsamples,
@@ -1028,7 +1001,11 @@ unsigned int pack_pcm(unsigned char *data, unsigned int nsamples,
 		      int resolution, unsigned long *clipped,
 		      mad_fixed_t *clipping)
 {
-  static mad_fixed_t left_err, right_err;
+# ifdef NEW_DITHER
+  static struct dither left_dither, right_dither;
+# else
+  static mad_fixed_t left_dither, right_dither;
+# endif
   unsigned char const *start;
   register signed long sample0, sample1;
   int effective, bytes;
@@ -1039,15 +1016,15 @@ unsigned int pack_pcm(unsigned char *data, unsigned int nsamples,
 
   if (right) {  /* stereo */
     while (nsamples--) {
-      sample0 = linear_dither(effective, *left++, &left_err,
+      sample0 = linear_dither(effective, *left++, &left_dither,
 			      clipped, clipping);
-      sample1 = linear_dither(effective, *right++, &right_err,
+      sample1 = linear_dither(effective, *right++, &right_dither,
 			      clipped, clipping);
 
       switch (resolution) {
       case 8:
-	data[0] = sample0 + 0x80;
-	data[1] = sample1 + 0x80;
+	data[0] = sample0 ^ 0x80;
+	data[1] = sample1 ^ 0x80;
 	break;
 
       case 32:
@@ -1070,12 +1047,12 @@ unsigned int pack_pcm(unsigned char *data, unsigned int nsamples,
   }
   else {  /* mono */
     while (nsamples--) {
-      sample0 = linear_dither(effective, *left++, &left_err,
+      sample0 = linear_dither(effective, *left++, &left_dither,
 			      clipped, clipping);
 
       switch (resolution) {
       case 8:
-	data[0] = sample0 + 0x80;
+	data[0] = sample0 ^ 0x80;
 	break;
 
       case 32:
@@ -1125,7 +1102,7 @@ int vbr_update(struct stats *stats, unsigned long bitrate)
 
   stats->bitrate = bitrate;
 
-  return stats->vbr ?
+  return (stats->vbr && stats->frames > 0) ?
     ((stats->vbr_rate * 2) / stats->frames + 1) / 2 : stats->bitrate;
 }
 
@@ -1134,6 +1111,7 @@ int do_error(struct mad_stream *stream, struct mad_frame *frame,
 	     struct input *input, int *last_error, struct stats *stats)
 {
   int do_continue = 1;
+  signed long tagsize;
 
   switch (stream->error) {
   case MAD_ERROR_BADCRC:
@@ -1153,23 +1131,10 @@ int do_error(struct mad_stream *stream, struct mad_frame *frame,
     break;
 
   case MAD_ERROR_LOSTSYNC:
-    if (strncmp(stream->this_frame, "ID3", 3) == 0) {
-      unsigned long tagsize;
-
-      /* ID3v2 tag */
-      if (id3v2_tag(stream->this_frame, stream->bufend - stream->this_frame,
-		    input, &tagsize) == -1)
-	return 1;
-
-      if (tagsize > stream->bufend - stream->this_frame)
-	mad_stream_skip(stream, stream->bufend - stream->this_frame);
-      else
-	mad_stream_skip(stream, tagsize);
-    }
-    else if (strncmp(stream->this_frame, "TAG", 3) == 0) {
-      /* ID3v1 tag */
-      mad_stream_skip(stream, 128);
-    }
+    tagsize = id3_tag_query(stream->this_frame,
+			    stream->bufend - stream->this_frame);
+    if (tagsize > 0)
+      mad_stream_skip(stream, tagsize);
     else if (stats)
       ++stats->sync_errors;
 
@@ -1194,7 +1159,7 @@ void do_vis(char *data, int nch, int resolution, int position)
 
   /*
    * Winamp visuals may have problems accepting sample sizes larger than
-   * 16 bits, so we reduce the sample size here.
+   * 16 bits, so we reduce the sample size here if necessary.
    */
 
   switch (resolution) {
@@ -1300,7 +1265,7 @@ DWORD WINAPI run_decode_thread(void *param)
       int nch, bytes;
       mad_fixed_t const *ch1, *ch2;
 
-      if (state->seek != -1 && state->length >= 0) {
+      if (state->seek != -1 && state->length > 0) {
 	int new_position;
 
 	if (state->seek < 0)
@@ -1313,8 +1278,7 @@ DWORD WINAPI run_decode_thread(void *param)
 
 	if (input_seek(&state->input, (double) new_position *
 		       state->size / state->length, FILE_BEGIN) != -1) {
-	  mad_timer_set(&timer, new_position / 1000,
-			new_position % 1000, 1000);
+	  mad_timer_set(&timer, 0, new_position, 1000);
 
 	  mad_frame_mute(&frame);
 	  mad_synth_mute(&synth);
@@ -1399,11 +1363,9 @@ DWORD WINAPI run_decode_thread(void *param)
 	switch (state->channel) {
 	case CHANNEL_RIGHT:
 	  ch1 = ch2;
-
 	case CHANNEL_LEFT:
 	  ch2 = 0;
 	  nch = 1;
-
 	case CHANNEL_MONO:
 	case CHANNEL_STEREO:
 	  break;
@@ -1501,75 +1463,6 @@ DWORD WINAPI run_decode_thread(void *param)
   return 0;
 }
 
-struct xing {
-  int flags;
-  unsigned long frames;
-  unsigned long bytes;
-  unsigned char toc[100];
-  long scale;
-};
-
-enum {
-  XING_FRAMES = 0x0001,
-  XING_BYTES  = 0x0002,
-  XING_TOC    = 0x0004,
-  XING_SCALE  = 0x0008
-};
-
-# define XING_MAGIC	(('X' << 24) | ('i' << 16) | ('n' << 8) | 'g')
-
-static
-int parse_xing(struct xing *xing, struct mad_bitptr ptr, unsigned int bitlen)
-{
-  if (bitlen < 64 || mad_bit_read(&ptr, 32) != XING_MAGIC)
-    goto fail;
-
-  xing->flags = mad_bit_read(&ptr, 32);
-  bitlen -= 64;
-
-  if (xing->flags & XING_FRAMES) {
-    if (bitlen < 32)
-      goto fail;
-
-    xing->frames = mad_bit_read(&ptr, 32);
-    bitlen -= 32;
-  }
-
-  if (xing->flags & XING_BYTES) {
-    if (bitlen < 32)
-      goto fail;
-
-    xing->bytes = mad_bit_read(&ptr, 32);
-    bitlen -= 32;
-  }
-
-  if (xing->flags & XING_TOC) {
-    int i;
-
-    if (bitlen < 800)
-      goto fail;
-
-    for (i = 0; i < 100; ++i)
-      xing->toc[i] = mad_bit_read(&ptr, 8);
-
-    bitlen -= 800;
-  }
-
-  if (xing->flags & XING_SCALE) {
-    if (bitlen < 32)
-      goto fail;
-
-    xing->scale = mad_bit_read(&ptr, 32);
-    bitlen -= 32;
-  }
-
-  return 0;
-
- fail:
-  xing->flags = 0;
-  return -1;
-}
-
 static
 int scan_header(struct input *input, struct mad_header *header,
 		struct xing *xing)
@@ -1583,6 +1476,9 @@ int scan_header(struct input *input, struct mad_header *header,
   mad_stream_init(&stream);
   mad_frame_init(&frame);
 
+  if (xing)
+    xing->flags = 0;
+
   while (1) {
     if (buflen < sizeof(buffer)) {
       DWORD bytes;
@@ -1609,7 +1505,7 @@ int scan_header(struct input *input, struct mad_header *header,
       }
 
       if (count++ ||
-	  (xing && parse_xing(xing, stream.anc_ptr, stream.anc_bitlen) == -1))
+	  (xing && xing_parse(xing, stream.anc_ptr, stream.anc_bitlen) == -1))
 	break;
     }
 
@@ -1815,11 +1711,14 @@ int play_stream(struct state *state)
 
   input_init(&state->input, INPUT_STREAM, stream);
 
-  if (scan_header(&state->input, &header, 0) == -1) {
+  if (scan_header(&state->input, &header, &state->xing) == -1) {
     input_close(&state->input);
 
+# if 0
     show_error(0, "Error Reading Stream", IDS_WARN_NOHEADER);
-    return 2;
+# endif
+
+    return -1;
   }
 
   state->size    = 0;
@@ -1836,7 +1735,6 @@ int play_file(struct state *state)
 {
   HANDLE file;
   struct mad_header header;
-  struct xing xing;
 
   file = CreateFile(state->path, GENERIC_READ,
 		    FILE_SHARE_READ | FILE_SHARE_WRITE, 0,
@@ -1850,16 +1748,19 @@ int play_file(struct state *state)
     show_error(0, "Error Opening File", error);
 # endif
 
-    return (error == ERROR_FILE_NOT_FOUND) ? -1 : 1;
+    return -1;
   }
 
   input_init(&state->input, INPUT_FILE, file);
 
-  if (scan_header(&state->input, &header, &xing) == -1) {
+  if (scan_header(&state->input, &header, &state->xing) == -1) {
     input_close(&state->input);
 
+# if 0
     show_error(0, "Error Reading File", IDS_WARN_NOHEADER);
-    return 2;
+# endif
+
+    return -1;
   }
 
   input_seek(&state->input, 0, FILE_BEGIN);
@@ -1867,16 +1768,16 @@ int play_file(struct state *state)
   state->size    = GetFileSize(file, 0);
   state->bitrate = 0;
 
-  if (xing.flags & XING_FRAMES) {
+  if (state->xing.flags & XING_FRAMES) {
     mad_timer_t timer;
 
     timer = header.duration;
-    mad_timer_multiply(&timer, xing.frames);
+    mad_timer_multiply(&timer, state->xing.frames);
 
     state->length = mad_timer_count(timer, MAD_UNITS_MILLISECONDS);
 
-    if (xing.flags & XING_BYTES)
-      state->bitrate = xing.bytes * 8 / state->length;
+    if ((state->xing.flags & XING_BYTES) && state->length > 0)
+      state->bitrate = state->xing.bytes * 8 / state->length;
   }
   else
     state->length = -(state->size * 8 / (header.bitrate / 1000));  /* est. */
@@ -2104,6 +2005,8 @@ struct fileinfo {
     struct mad_header header;
   } mpeg;
 
+  struct id3_file *id3file;
+
   struct {
     int has;
     struct id3v1 tag;
@@ -2306,10 +2209,8 @@ BOOL mpeg_dialog(HWND dialog, UINT message,
 	SetDlgItemText(dialog, IDC_MPEG_LAYER, ptr);
 
 	sprintf(str, "%lu kbps", header->bitrate / 1000);
-# if 0  /* not enough room for this in the dialog field */
 	if (header->flags & MAD_FLAG_FREEFORMAT)
 	  strcat(str, " (free format)");
-# endif
 	SetDlgItemText(dialog, IDC_MPEG_BITRATE, str);
 
 	sprintf(str, "%u Hz", header->samplerate);
@@ -2335,19 +2236,19 @@ BOOL mpeg_dialog(HWND dialog, UINT message,
 
 	switch (header->mode) {
 	case MAD_MODE_SINGLE_CHANNEL:
-	  CheckDlgButton(dialog, IDC_MPEG_SINGLECH, BST_CHECKED);
+	  SetDlgItemText(dialog, IDC_MPEG_MODE, "Single Channel");
 	  break;
 
 	case MAD_MODE_DUAL_CHANNEL:
-	  CheckDlgButton(dialog, IDC_MPEG_DUALCH, BST_CHECKED);
+	  SetDlgItemText(dialog, IDC_MPEG_MODE, "Dual Channel");
 	  break;
 
 	case MAD_MODE_JOINT_STEREO:
-	  CheckDlgButton(dialog, IDC_MPEG_JOINTST, BST_CHECKED);
+	  SetDlgItemText(dialog, IDC_MPEG_MODE, "Joint Stereo");
 	  break;
 
 	case MAD_MODE_STEREO:
-	  CheckDlgButton(dialog, IDC_MPEG_STEREO, BST_CHECKED);
+	  SetDlgItemText(dialog, IDC_MPEG_MODE, "Stereo");
 	  break;
 	}
 
@@ -2359,8 +2260,7 @@ BOOL mpeg_dialog(HWND dialog, UINT message,
 	  CheckDlgButton(dialog, IDC_MPEG_CRC, BST_CHECKED);
       }
       else {
-	mad_timer_set(&header->duration, info->mpeg.length / 1000,
-		      info->mpeg.length % 1000, 1000);
+	mad_timer_set(&header->duration, 0, info->mpeg.length, 1000);
 	mad_timer_string(header->duration, str,
 			 "%lu:%02u:%02u.%1u", MAD_UNITS_HOURS,
 			 MAD_UNITS_DECISECONDS, 0);
@@ -2370,8 +2270,7 @@ BOOL mpeg_dialog(HWND dialog, UINT message,
 	SetDlgItemText(dialog, IDC_MPEG_LENGTH, str);
 
 	if (info->mpeg.bitrate != header->bitrate / 1000) {
-	  sprintf(str, "%d kbps", info->mpeg.bitrate);
-	  SetDlgItemText(dialog, IDC_MPEG_BITRATELABEL, "Avg. Bitrate:");
+	  sprintf(str, "%d kbps (average)", info->mpeg.bitrate);
 	  SetDlgItemText(dialog, IDC_MPEG_BITRATE, str);
 	}
       }
@@ -2686,8 +2585,12 @@ BOOL stats_dialog(HWND dialog, UINT message,
 
 	/* Output */
 
-	db = 20 * log10(mad_f_todouble(state.attenuation));
-	sprintf(str, "%.1f dB", db);
+	if (state.attenuation == 0)
+	  strcpy(str, "-inf dB");
+	else {
+	  db = 20 * log10(mad_f_todouble(state.attenuation));
+	  sprintf(str, "%.1f dB", db);
+	}
 	SetDlgItemText(dialog, IDC_STATS_ATTENUATION, str);
 
 	SetDlgItemInt(dialog, IDC_STATS_CLIPPED, stats->clipped, FALSE);
@@ -2802,9 +2705,10 @@ void proppage_init(PROPSHEETPAGE *page, int proppage_id,
 static
 int show_infobox(char *path, HWND parent)
 {
+  struct fileinfo info;
+  int fd;
   PROPSHEETPAGE psp[4];
   PROPSHEETHEADER psh;
-  struct fileinfo info;
   DWORD bytes;
 
   if (is_stream(path)) {
@@ -2816,7 +2720,9 @@ int show_infobox(char *path, HWND parent)
   info.file = CreateFile(path, GENERIC_READ | GENERIC_WRITE,
 			 FILE_SHARE_READ, 0,
 			 OPEN_EXISTING, FILE_FLAG_RANDOM_ACCESS, 0);
-  if (info.file == INVALID_HANDLE_VALUE) {
+  if (info.file != INVALID_HANDLE_VALUE)
+    info.attributes = 0;
+  else {
     info.file = CreateFile(path, GENERIC_READ,
 			   FILE_SHARE_READ | FILE_SHARE_WRITE, 0,
 			   OPEN_EXISTING, FILE_FLAG_RANDOM_ACCESS, 0);
@@ -2827,8 +2733,6 @@ int show_infobox(char *path, HWND parent)
 
     info.attributes = FILE_ATTRIBUTE_READONLY;
   }
-  else
-    info.attributes = 0;
 
   info.size = GetFileSize(info.file, 0);
 
@@ -2846,6 +2750,24 @@ int show_infobox(char *path, HWND parent)
   info.mpeg.bitrate = 0;
   mad_header_init(&info.mpeg.header);
 
+  fd = _open_osfhandle((long) info.file,
+		       ((info.attributes & FILE_ATTRIBUTE_READONLY) ?
+			_O_RDONLY : _O_RDWR) | _O_BINARY);
+  if (fd == -1) {
+    show_error(parent, "Error Converting File Handle", GetLastError());
+    CloseHandle(info.file);
+    return 1;
+  }
+
+  info.id3file =
+    id3_file_fdopen(fd, (info.attributes & FILE_ATTRIBUTE_READONLY) ?
+		    ID3_FILE_MODE_READONLY : ID3_FILE_MODE_READWRITE);
+  if (info.id3file == 0) {
+    _close(fd);
+    show_error(parent, "ID3 Error", IDS_ERR_ID3FAIL);
+    return 1;
+  }
+
   SetFilePointer(info.file, -sizeof(info.id3v1.tag.data), 0, FILE_END);
   if (ReadFile(info.file, info.id3v1.tag.data,
 	       sizeof(info.id3v1.tag.data), &bytes, 0) == 0)
@@ -2877,7 +2799,7 @@ int show_infobox(char *path, HWND parent)
 
   PropertySheet(&psh);
 
-  CloseHandle(info.file);
+  id3_file_close(info.id3file);
 
   return 0;
 }
